# Generated by Django 2.2.12 on 2020-06-15 09:32

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('wikijs', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='WikiJsUser',
            new_name='WikiJs',
        ),
    ]
